import java.awt.*;
import java.awt.event.*; 
import java.awt.image.BufferedImage; 
import javax.swing.*;


/**
 * Developed for 2009/2010 RSGC ICS4U ACES: <a
 * href="http://darcy.rsgc.on.ca/ACES/ICS4U/Tasks/0910Tasks.htm#Koch"
 * >Plane-Filling Curves Assignment</a>.<br>
 * <b>Date: 2009 11 12</b>
 * 
 * @author C. D'Arcy
 */
public class LSystems extends JApplet implements ActionListener {
  // The program's base dimensions.
  private static Dimension appDimen = new Dimension(200, 200);
  // The singular panel onto which will be drawn the individual monsters
  static Content content;

  public static void main(String[] args) {
    JFrame frame = new JFrame();

    frame.setTitle("Plane-Filling Curves");
    // end program when window is closed
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    });

    JApplet applet = new LSystems();

    applet.init();
    GraphicsConfiguration gc = frame.getGraphicsConfiguration();
    Rectangle rect = gc.getBounds();

    frame.setLocation((rect.width - appDimen.width) / 2,
        (rect.height - appDimen.height) / 2);
    frame.getContentPane().add(applet);
    frame.pack();
    frame.setVisible(true);

  }

  public LSystems() {}

  public void init() {
    JMenuBar mb = new JMenuBar();

    setJMenuBar(mb);
    JMenu menu;

    /*
     * menu = new JMenu("File"); mb.add(menu); addMenuItem(menu, "Exit",
     * "Exit", this);
     */
    menu = new JMenu("Koch");
    mb.add(menu);

    addMenuItem(menu, "Figure 1.7 a)", "RBK1", this);
    addMenuItem(menu, "Figure 1.9 a)", "IC1", this);
    addMenuItem(menu, "Figure 1.9 b)", "OE1", this);
    addMenuItem(menu, "Figure 1.9 c)", "MI1", this);
    addMenuItem(menu, "Figure 1.9 f)", "MK1", this);
    menu = new JMenu("L-Sys");
    mb.add(menu);
    menu = new JMenu("Bracketed");
    mb.add(menu);
    menu = new JMenu("3D");
    mb.add(menu);
    content = new Content();
    getContentPane().add(content);
  }

  public void start() {
    content.createNewImage();
    content.repaint();
  }

  public void addMenuItem(JMenu menu, String label, String action,
      LSystems app) {
    JMenuItem mi = new JMenuItem(label);
    mi.setActionCommand(action);
    menu.add(mi);
    mi.addActionListener(app);
  }

  public void actionPerformed(ActionEvent ev) {
    String command = ev.getActionCommand();

    if ("Exit".equals(command)) {
      System.exit(0);
    } else if ("RBK1".equals(command)) {
      content.current = content.new Koch("Triadic Koch Snowflake", "F--F--F",
          "F", new String[] { "F+F--F+F" }, 0, 150, 60, 3, 3,
          -appDimen.width / 3, -appDimen.height / 6);

    } else if ("IC1".equals(command)) {
      content.current = content.new Koch("Triadic Koch Snowflake", "F--F--F",
          "F", new String[] { "F+F--F+F" }, 0, 150, 60, 3, 3,
          -appDimen.width / 3, -appDimen.height / 6);

    } else if ("OE1".equals(command)) {
      content.current = content.new Koch("Triadic Koch Snowflake", "F--F--F",
          "F", new String[] { "F+F--F+F" }, 0, 150, 60, 3, 3,
          -appDimen.width / 3, -appDimen.height / 6);

    } else if ("MI1".equals(command)) {
      content.current = content.new Koch("Triadic Koch Snowflake", "F--F--F",
          "F", new String[] { "F+F--F+F" }, 0, 150, 60, 3, 3,
          -appDimen.width / 3, -appDimen.height / 6);

    } else if ("MK1".equals(command)) {
      content.current = content.new Koch("Triadic Koch Snowflake", "F--F--F",
          "F", new String[] { "F+F--F+F" }, 0, 150, 60, 3, 3,
          -appDimen.width / 3, -appDimen.height / 6);

    }
    content.createNewImage();
    content.repaint();

  }

  /**
   * A JPanel class used to as a container for all L-System renderings.
   * 
   * @author C. D'Arcy (based on Ireland Comery's earlier Parametric Equation
   *         assignment)
   */
  public class Content extends JPanel {
    // The temporary buffered image used for drawing off screen...
    private BufferedImage img;
    // The reference to img's drawing canvas...
    private Graphics2D drawImg;
    // The active plane-filling curve...
    private Koch current;

    // Canvas colours...
    private Color backgroundColor;
    private Color curveColor;

    /**
     * Constructor initializes
     * <ol>
     * <li>the Triadic Koch Snowflake as the initial Monster object</li>
     * <li>the active Canvas colours</li>
     * <li>the offscreen BufferedImage</li>
     * </ol>
     */
    public Content() {
      current = new Koch("Triadic Koch Snowflake", "F--F--F", "F",
          new String[] { "F+F--F+F" }, 0, 150, 60, 3, 3, -appDimen.width / 3,
          -appDimen.height / 6);

      backgroundColor = new Color(0, 0, 128); // navy
      curveColor = Color.white;
      img = new BufferedImage(appDimen.width, appDimen.height,
          BufferedImage.TYPE_INT_RGB);

      drawImg = img.createGraphics();

      setPreferredSize(appDimen);
    }

    /**
     * Creates and sets up a new <code>BufferedImage</code> with
     * <code>this component</code>'s size, and draws the current Monster
     * curve onto that <code>BufferedImage</code>.
     */
    public void createNewImage() {

      try {

        img = new BufferedImage(getWidth(), getHeight(),
            BufferedImage.TYPE_INT_RGB);
        drawImg = img.createGraphics();
        drawImg.setColor(curveColor);
        drawImg.setBackground(backgroundColor);
        drawImg.clearRect(0, 0, getWidth(), getHeight());

        drawImg.translate(getWidth() / 2, getHeight() / 2);
        drawImg.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);

        current.draw(drawImg);
      } catch (IllegalArgumentException e) {}
    }

    /**
     * Returns the main <code>BufferedImage</code> being used by the
     * <code>paintComponent</code> method.
     * 
     * @return the <code>BufferedImage</code> currently being used as a
     *         graphic.
     */
    public BufferedImage getImage() {
      return img;
    }

    /**
     * Overridden <code>paintComponent</code> method, to draw to
     * <code>this</code> component.
     * 
     * @param g
     *            a graphic to draw on.
     */
    public void paintComponent(Graphics g) {
      super.paintComponent(g);
      Graphics2D g2D = (Graphics2D) g;
      g2D.drawImage(getImage(), 0, 0, this);
     }

    /**
     * This class encapsulates the details and drawing of the Koch category of
     * plane-filling curves.
     */
    public class Koch {

      /**
       * The name of this instance of the Koch object
       */
      public String name;

      /** the initiator (axiom) of the fractal */
      public String axiom;

      /** holds the characters for which productions exist */
      public String keys;

      /**
       * Holds the characters defining the fractals as a sequence of
       * drawing commands for the Turtle
       */
      public String definition;

      /**
       * The array of productions (generators) that replace their
       * respective key in each generation
       */
      public String[] productions;

      /** The initial angle of the Turtle in degrees */
      public double heading;

      /**
       * The distance or length of each line segment(decreases with each
       * generation).
       */
      public int distance;

      /** The angle the heading of the turtle is modified by at each turn */
      public double delta;

      /** the factor used to decrease the distance with each generation */
      public int factor;

      /** The number of generations to propagate. */
      public int n;

      /**
       * The initial x position of the turtle
       */
      public int xPos;

      /**
       * The initial y position of the turtle
       */
      public int yPos;

      /**
       * Constructor for basic Koch curve: one key 
       * @param name the name of the Koch curve
       * @param axiom the initial (generation 0) curve (the initiator in Koch parlance)
       * @param keys the String containing the keys. Koch curve only has one key but keep as a String for future development
       * @param productions the array of productions. Koch curve only has one production but keep as an array of String for future development
       * @param heading the initial heading of the drawing pen (turtle) in degrees. 0 degrees is east. Positive is Counterclockwise. 
       * @param distance the length of the line to draw in pixels when moving the turtle forward (the F command)
       * @param delta the change in the turtle's heading when executing + or - command. + is counterclockwise and - is clockwise 
       * @param factor the reduction ratio applied to distance after each generation
       * @param n the number of generations to propagate the curve
       * @param xPos the initial x position (column) of the turtle 
       * @param yPos the initial y position (row) of the turtle
       */
		
      public Koch(String name, String axiom, String keys,
          String[] productions, double heading, int distance,
          double delta, int factor, int n, int xPos, int yPos) {
      }

    } //Koch

  }  //Content

}  //LSystems
